/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.springframework.data.repository.history;

import java.io.Serializable;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.history.Revision;
import org.springframework.data.history.Revisions;
import org.springframework.data.repository.NoRepositoryBean;

/**
 * A repository which can access entities held in a variety of {@link Revisions}.
 * 
 * @author Oliver Gierke
 * @author Philipp Huegelmeyer
 */
@NoRepositoryBean
public interface RevisionRepository<T, ID extends Serializable, N extends Number & Comparable<N>> {

	/**
	 * Returns the revision of the entity it was last changed in.
	 * 
	 * @param id must not be {@literal null}.
	 * @return
	 */
	Revision<N, T> findLastChangeRevision(ID id);

	/**
	 * Returns all {@link Revisions} of an entity with the given id.
	 * 
	 * @param id must not be {@literal null}.
	 * @return
	 */
	Revisions<N, T> findRevisions(ID id);

	/**
	 * Returns a {@link Page} of revisions for the entity with the given id.
	 * 
	 * @param id must not be {@literal null}.
	 * @param pageable
	 * @return
	 */
	Page<Revision<N, T>> findRevisions(ID id, Pageable pageable);
}
